"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const lodash_flatten_1 = __importDefault(require("lodash.flatten"));
const lodash_frompairs_1 = __importDefault(require("lodash.frompairs"));
const debug_1 = __importDefault(require("./lib/debug"));
const extract_locale_1 = __importDefault(require("./lib/extract-locale"));
const resolve_pattern_1 = __importDefault(require("./lib/resolve-pattern"));
exports.types = __importStar(require("./types"));
const pluginName = 'HtmlWebpackInlineI18nPlugin';
class HtmlWebpackInlineI18nPlugin {
    constructor(options = {}) {
        var _a, _b, _c;
        this.options = options;
        this.options.include = (_a = options.include) !== null && _a !== void 0 ? _a : /\.i18n/iu;
        this.options.position = (_b = options.position) !== null && _b !== void 0 ? _b : 'before';
        this.options.extractLocale = (_c = options.extractLocale) !== null && _c !== void 0 ? _c : extract_locale_1.default;
    }
    apply(compiler) {
        compiler.hooks.compilation.tap(pluginName, (compilation) => {
            debug_1.default('Started apply.compiler.compilation');
            let i18nAssetTag = {
                tagName: 'div',
                voidTag: false,
                attributes: { id: 'i18n' },
            };
            const { beforeAssetTagGeneration, alterAssetTagGroups, } = html_webpack_plugin_1.default.getHooks(compilation);
            beforeAssetTagGeneration.tapPromise(pluginName, (data) => __awaiter(this, void 0, void 0, function* () {
                debug_1.default('Started hook "beforeAssetTagGeneration"');
                const { include, exclude, modifyTag, extractLocale } = this.options;
                const i18nEntryGroups = compilation.chunks.map((chunk) => chunk.files
                    .filter((filename) => {
                    const ctx = Object.assign(Object.assign({}, data), { chunk, filename });
                    const [inclPattern, exclPattern] = [
                        ['include', include],
                        ['exclude', exclude],
                    ].map(([name, pattern]) => pattern && resolve_pattern_1.default(name, pattern, ctx));
                    return ((inclPattern ? filename.match(inclPattern) : true) &&
                        (exclPattern ? !filename.match(exclPattern) : true));
                })
                    .map((filename) => {
                    const ctx = Object.assign(Object.assign({}, data), { chunk, filename });
                    const pattern = resolve_pattern_1.default('extractLocale', extractLocale, ctx);
                    const match = filename.match(pattern);
                    const locale = match
                        ? match.groups
                            ? match.groups.locale
                            : match[1]
                        : 'default';
                    return [`data-${locale}`, filename];
                }));
                const i18n = lodash_frompairs_1.default(lodash_flatten_1.default(i18nEntryGroups).sort(([a], [b]) => a.localeCompare(b)));
                Object.assign(i18nAssetTag.attributes, i18n);
                if (modifyTag) {
                    debug_1.default('Started function "modifyTag"');
                    i18nAssetTag = yield modifyTag(Object.assign(Object.assign({}, data), { i18nTag: i18nAssetTag }));
                    debug_1.default('Done function "modifyTag"');
                }
                debug_1.default('Done hook "beforeAssetTagGeneration"');
                return data;
            }));
            alterAssetTagGroups.tapPromise(pluginName, (data) => __awaiter(this, void 0, void 0, function* () {
                debug_1.default('Started hook "alterAssetTagGroups"');
                const { position } = this.options;
                const { bodyTags = [] } = data;
                data.bodyTags =
                    typeof position === 'function'
                        ? position(Object.assign(Object.assign({}, data), { i18nTag: i18nAssetTag }))
                        : position === 'before'
                            ? [i18nAssetTag, ...bodyTags]
                            : [...bodyTags, i18nAssetTag];
                debug_1.default('Done hook "alterAssetTagGroups"');
                return data;
            }));
            debug_1.default('Done apply.compiler.compilation');
        });
    }
}
exports.default = HtmlWebpackInlineI18nPlugin;
